if select(2, UnitClass("player")) ~= "WARLOCK" then return end
local GT = LibStub:GetLibrary("LibGratuity-3.0")
local PlayerAura = DrDamage.PlayerAura

local UnitBuff = UnitBuff
local UnitCreatureFamily = UnitCreatureFamily
local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
local UnitStat = UnitStat

--Downranking exceptions: Death Coil, Siphon life, Searing Pain, Hellfire
--Untested assumed normal: UA, Conflagrate, Shadowfury
--No downranking: Lifetap, Drain soul, CoA

function DrDamage:PlayerData()

	-- SHADOW BITE effected by presence of spells designated FEED SHADOW BITE
	self.Calculation["Shadow Bite"] = function( calculation, ActiveAuras, _ )
		if ActiveAuras["Feed Shadow Bite"] then
			local SSBonus = 0
			SBBonus = ActiveAuras["Feed Shadow Bite"] * 0.05
			calculation.minDam = calculation.minDam * ( 1 + SSBonus )	
			calculation.maxDam = calculation.maxDam * ( 1 + SSBonus )	
		end
	end

	-- DRAIN LIFE effected by presence of spells accumulating SOUL SIPHON effect
	self.Calculation["Drain Life"] = function( calculation, ActiveAuras, BuffTalentRanks )
		if ActiveAuras["Soul Siphon"] and BuffTalentRanks["Soul Siphon"] then
			local SSBonus = 0
			if BuffTalentRanks["Soul Siphon"] == 1 then
				SSBonus = ActiveAuras["Soul Siphon"] * 0.02
				if SSBonus > 0.24 then SSBonus = 0.24 end
			elseif BuffTalentRanks["Soul Siphon"] == 2 then
				SSBonus = ActiveAuras["Soul Siphon"] * 0.04
				if SSBonus > 0.60 then SSBonus = 0.60 end
			end
			calculation.dmgM = calculation.dmgM * ( 1 + SSBonus )	
		end
	end

	-- DRAIN LIFE effected by DEATH'S EMBRACE effect
	self.Calculation["Drain Life"] = function( calculation, ActiveAuras, BuffTalentRanks )
		if BuffTalentRanks["Death's Embrace"] and (UnitHealth("player") / UnitHealthMax("player") <= 0.2 ) then
			calculation.dmgM = calculation.dmgM * ( 1 + 0.1 * BuffTalentRanks["Death's Embrace"] )	
		end
	end

	-- SHADOW spells effected by DEATH'S EMBRACE EFFECT
	self.Calculation["Shadow"] = function( calculation, ActiveAuras, BuffTalentRanks )
		if BuffTalentRanks["Death's Embrace"] and (UnitHealth("target") / UnitHealthMax("target") <= 0.35 ) then
			calculation.dmgM = calculation.dmgM * ( 1 + 0.04 * BuffTalentRanks["Death's Embrace"] )	
		end
	end

	-- SHADOW spells effected by TOUCH OF SHADOW
	self.Calculation["Shadow"] = function( calculation, ActiveAuras, BuffTalentRanks )
		if ActiveAuras["Touch of Shadow"] then
			calculation.dmgM = calculation.dmgM * 1.1	
		end
	end

	-- INCINERATE effected by IMMOLATE
	self.Calculation["Incinerate"] = function( calculation, ActiveAuras, _, spell )
		if ActiveAuras["Immolate"] then
			calculation.minDam = calculation.minDam + spell.incinerateMin
			calculation.maxDam = calculation.maxDam + spell.incinerateMax
		end
	end
	
	-- IMPROVED SHADOW BOLT effects all SHADOW spells
	self.Calculation["Shadow"] = function( calculation, ActiveAuras, BuffTalentRanks )
		if ActiveAuras["Shadow Mastery"] then
			calculation.dmgM = calculation.dmgM * ( 1 + 0.2 * BuffTalentRanks["Improved Shadow Bolt"] )	
		end
	end

	-- PANDEMIC effects CORRUPTION and UNSTABLE AFFLICTION
	self.Calculation["Corruption"] = function( calculation, ActiveAuras, BuffTalentRanks)
		if BuffTalentRanks["Pandemic"] == 3 then
			calculation.critM = 1
		elseif BuffTalentRanks["Pandemic"] == 2 then
			calculation.critM = 0.66
		elseif BuffTalentRanks["Pandemic"] == 1 then
			calculation.critM = 0.33
		else
			calculation.critM = 0
		end
	end
	self.Calculation["Unstable Affliction"] = function( calculation, ActiveAuras, BuffTalentRanks )
		if BuffTalentRanks["Pandemic"] == 3 then
			calculation.critM = 1
		elseif BuffTalentRanks["Pandemic"] == 2 then
			calculation.critM = 0.66
		elseif BuffTalentRanks["Pandemic"] == 1 then
			calculation.critM = 0.33
		else
			calculation.critM = 0
		end
	end

	-- SHADOW EMBRACE effects all periodic damage spells
	self.Calculation["Corruption"] = function( calculation, ActiveAuras, BuffTalentRanks)
		if PlayerAura["Shadow Embrace"] then
			calculation.dmgM = calculation.dmgM * (1 + 0.01 * BuffTalentRanks["Shadow Embrace"])	
		end
	end
	self.Calculation["Curse of Agony"] = function( calculation, ActiveAuras, BuffTalentRanks )
		if PlayerAura["Shadow Embrace"] then
			calculation.dmgM = calculation.dmgM * (1 + 0.01 * BuffTalentRanks["Shadow Embrace"])	
		end
	end
	self.Calculation["Curse of Doom"] = function( calculation, ActiveAuras, BuffTalentRanks )
		if PlayerAura["Shadow Embrace"] then
			calculation.dmgM = calculation.dmgM * (1 + 0.01 * BuffTalentRanks["Shadow Embrace"])
		end
	end
	self.Calculation["Drain Life"] = function( calculation, ActiveAuras, BuffTalentRanks )
		if PlayerAura["Shadow Embrace"] then
			calculation.dmgM = calculation.dmgM * (1 + 0.01 * BuffTalentRanks["Shadow Embrace"])
		end
	end
	self.Calculation["Drain Soul"] = function( calculation, ActiveAuras, BuffTalentRanks )
		if PlayerAura["Shadow Embrace"] then
			calculation.dmgM = calculation.dmgM * (1 + 0.01 * BuffTalentRanks["Shadow Embrace"])
		end
	end
	self.Calculation["Seed of Corruption"] = function( calculation, ActiveAuras, BuffTalentRanks )
		if PlayerAura["Shadow Embrace"] then
			calculation.dmgM = calculation.dmgM * (1 + 0.01 * BuffTalentRanks["Shadow Embrace"])
		end
	end
	self.Calculation["Siphon Life"] = function( calculation, ActiveAuras, BuffTalentRanks )
		if PlayerAura["Shadow Embrace"] then
			calculation.dmgM = calculation.dmgM * (1 + 0.01 * BuffTalentRanks["Shadow Embrace"])
		end
	end
	self.Calculation["Unstable Affliction"] = function( calculation, ActiveAuras, BuffTalentRanks )
		if PlayerAura["Shadow Embrace"] then
			calculation.dmgM = calculation.dmgM * (1 + 0.01 * BuffTalentRanks["Shadow Embrace"])
		end
	end

	--Set bonuses
	--Sets
	self.SetBonuses["Oblivion Raiment"] = { 27537, 27778, 27948, 28232, 28415 }
	self.SetBonuses["Voidheart Raiment"] = { 28963, 28964, 28966, 28967, 28968 }
	self.SetBonuses["Corruptor Raiment"] = { 30211, 30212, 30213, 30214, 30215 }
	self.SetBonuses["Plagueheart Raiment"] = { 22504, 22505, 22506, 22507, 22508, 22509, 22510, 22511, 23063 }
	self.SetBonuses["Malefic Raiment"] = { 31050, 31051, 31052, 31053, 31054, 34436, 34541, 34564 }
	
	--Effects
	self.SetBonuses["Seed of Corruption"] = function( calculation )
		if self:GetSetAmount( "Oblivion Raiment" ) >= 4 then
			calculation.finalMod = calculation.finalMod + 180
		end
	end	
	self.SetBonuses["Corruption"] = function( calculation )
		if self:GetSetAmount( "Voidheart Raiment" ) >= 4 then			
			calculation.eDuration = calculation.eDuration + 3
		end
		if self:GetSetAmount( "Plagueheart Raiment" ) >= 4 then
			calculation.dmgM = calculation.dmgM * 1.12
		end		
	end
	self.SetBonuses["Immolate"] = function( calculation )
		if self:GetSetAmount( "Voidheart Raiment" ) >= 4 then			
			calculation.eDuration = calculation.eDuration + 3
		end
	end
	self.SetBonuses["Shadow Bolt"] = function( calculation )
		if self:GetSetAmount( "Malefic Raiment" ) >= 4 then
			calculation.dmgM = calculation.dmgM * 1.06
		end		
	end
	self.SetBonuses["Incinerate"] = function( calculation )
		if self:GetSetAmount( "Malefic Raiment" ) >= 4 then
			calculation.dmgM = calculation.dmgM * 1.06
		end		
	end

	--AURA
	--Demon Armor
	self.PlayerAura[GetSpellInfo(706)] = { ModType = "Special" }
	--Fel Armor
	self.PlayerAura[GetSpellInfo(28176)] = { ModType = "Special" }	
	--Burning Wish
	self.PlayerAura[GetSpellInfo(18789)] = { School = "Fire", Value = 0.1 }
	--Touch of Shadow (Succubus)
	self.PlayerAura[GetSpellInfo(18791)] = { ModType = "Amount", Value = "Touch of Shadow" }
--	--Touch of Shadow (Felguard) --unnecessary (automatically taken into account)
--	self.PlayerAura[GetSpellInfo(35701)] = { ModType = "Special" }
	--Metamorphosis
	self.PlayerAura[GetSpellInfo(59672)] = { Effect = 0.2, Spells = "All", Add = true, }
	--Master Demonologist
	self.PlayerAura[GetSpellInfo(23785)] = { ModType = "Special" }

	--IMMOLATE
	--Incinerate
	self.TargetAura[GetSpellInfo(348)] = { Spell = GetSpellInfo(29722), ModType = "ActiveAura", ActiveAura = "Immolate", }

	--SHADOW MASTERY (Improved Shadow Bolt)
	--Shadow Bolt
	self.TargetAura[GetSpellInfo(17794)] = { Spell = GetSpellInfo(686), ModType = "ActiveAura", ActiveAura = "Shadow Mastery", }

	--SIPHON SOUL
	--Corruption
	self.TargetAura[GetSpellInfo(172)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	--Death Coil
	self.TargetAura[GetSpellInfo(6789)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	--Curse of Agony
	self.TargetAura[GetSpellInfo(980)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	--Curse of Doom
	self.TargetAura[GetSpellInfo(603)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	--Curse of Recklessness
	self.TargetAura[GetSpellInfo(704)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	--Curse of Weakness
	self.TargetAura[GetSpellInfo(702)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	--Curse of Tongues
	self.TargetAura[GetSpellInfo(1714)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	--Curse of Exhaustion
	self.TargetAura[GetSpellInfo(18223)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	--Fear
	self.TargetAura[GetSpellInfo(5782)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	--Drain Soul
	self.TargetAura[GetSpellInfo(1120)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	--Drain Mana
	self.TargetAura[GetSpellInfo(5138)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	--Siphon Life
	self.TargetAura[GetSpellInfo(18265)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	--Unstable Affliction
	self.TargetAura[GetSpellInfo(30108)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	--Seed of Corruption
	self.TargetAura[GetSpellInfo(27243)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	--Howl of Terror
	self.TargetAura[GetSpellInfo(5484)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	
	--SHADOW BITE (Felhunter)
	--Corruption
	self.TargetAura[GetSpellInfo(172)] = 	{ ActiveAura = "Feed Shadow Bite", Spell = GetSpellInfo(54049), ModType = "ActiveAura" }
	--Curse of Agony
	self.TargetAura[GetSpellInfo(980)] = 	{ ActiveAura = "Feed Shadow Bite", Spell = GetSpellInfo(54049), ModType = "ActiveAura" }
	--Curse of Doom
	self.TargetAura[GetSpellInfo(603)] = 	{ ActiveAura = "Feed Shadow Bite", Spell = GetSpellInfo(54049), ModType = "ActiveAura" }
	--Drain Life
	self.TargetAura[GetSpellInfo(689)] = 	{ ActiveAura = "Feed Shadow Bite", Spell = GetSpellInfo(54049), ModType = "ActiveAura" }
	--Drain Soul
	self.TargetAura[GetSpellInfo(1120)] = 	{ ActiveAura = "Feed Shadow Bite", Spell = GetSpellInfo(54049), ModType = "ActiveAura" }
	--Siphon Life
	self.TargetAura[GetSpellInfo(18265)] = 	{ ActiveAura = "Feed Shadow Bite", Spell = GetSpellInfo(54049), ModType = "ActiveAura" }
	--Unstable Affliction
	self.TargetAura[GetSpellInfo(30108)] = 	{ ActiveAura = "Feed Shadow Bite", Spell = GetSpellInfo(54049), ModType = "ActiveAura" }
	--Seed of Corruption
	self.TargetAura[GetSpellInfo(27243)] = 	{ ActiveAura = "Feed Shadow Bite", Spell = GetSpellInfo(54049), ModType = "ActiveAura" }
	
	--FEL ARMOR effects
	local felarmor = GetSpellInfo(28176)
	self.Calculation[GetSpellInfo(28176)] = function( calculation, _, BuffTalentRanks )
		if not UnitBuff("player", felarmor) then
			calculation.spellDmg = calculation.spellDmg + 100 * (1 + 0.1 * (BuffTalentRanks["Demonic Aegis"] or 0)) + 0.3 * select(2, UnitStat("player", 5))
		end
	end
	
	--DEMON ARMOR effects
	local demonarmor = GetSpellInfo(706)
	self.Calculation[GetSpellInfo(706)] = function( calculation, _, BuffTalentRanks )
		calculation.leechBonus = calculation.leechBonus  * (1.2 + 0.02 * (BuffTalentRanks["Demonic Aegis"] or 0))
	end

	--MASTER DEMONOLOGIST
	self.Calculation[GetSpellInfo(23785)] = function(calculation, _, BuffTalentRanks, spell)
--		if UnitCreatureFamily == "Felhunter" then --doesn't effect outgoing effects
--		if UnitCreatureFamily == "Felguard" then --unnecessary (automatically taken into account)
		if UnitCreatureFamily == "Imp" then
			for k,v in (spell.School) do
				if v == "Fire" then
					calculation.dmgM = calculation.dmgM * (1 + 0.01 * BuffTalentRanks["Master Demonologist"])
				end
			end
		elseif UnitCreatureFamily == "Succubus" then
			for k,v in (spell.School) do
				if v == "Shadow" then
					calculation.dmgM = calculation.dmgM * (1 + 0.01 * BuffTalentRanks["Master Demonologist"])
				end
			end
--		elseif UnitCreatureFamily == "Voidwalker" then --doesn't effect outgoing effects
		end
	end

	self.spellInfo = {
		-- WARLOCK abilities
		[GetSpellInfo(50796)] = {
			["Name"] = "Chaos Bolt",
			[0] = { School = { "Fire", "Destruction" }, castTime = 2, canCrit = true, Cooldown = 12, BaseIncrease = true, },
			[1] = { 607, 796, 1, 2, spellLevel = 60, }, --guessing level increase
			[2] = { 781, 991, 1, 2, spellLevel = 70, }, --guessing level increase
			[3] = { 882, 1120, 1, 2, spellLevel = 75, }, --guessing level increase
			[4] = { 1036, 1314, 1, 2, spellLevel = 80, }, --guessing level increase
		},
		[GetSpellInfo(172)] = {
			["Name"] = "Corruption",
			[0] = { School = { "Shadow", "Affliction", }, bonusFactor = 0.93, castTime = 0, eDot = true, eDuration = 18, sTicks = 3, CastMod = 0.1, canDotCrit = true },
			[1] = { 40, 40, spellLevel = 4, eDuration = 12, Downrank = -1, },
			[2] = { 90, 90, spellLevel = 14, eDuration = 15, },
			[3] = { 222, 222, spellLevel = 24, },
			[4] = { 324, 324, spellLevel = 34, },
			[5] = { 486, 486, spellLevel = 44, }, 
			[6] = { 666, 666, spellLevel = 54, },
			[7] = { 822, 822, spellLevel = 60, },
			[8] = { 900, 900, spellLevel = 65, },
			[9] = { 984, 984, spellLevel = 71, },
			[10] = { 1080, 1080, spellLevel = 77, },
		},
		[GetSpellInfo(980)] = {
			["Name"] = "Curse of Agony",
			[0] = { School = { "Shadow", "Affliction", }, bonusFactor = 1.20, eDot = true, eDuration = 24, },
			[1] = { 84, 84, spellLevel = 8,  },
			[2] = { 180, 180, spellLevel = 18, },
			[3] = { 324, 324, spellLevel = 28, }, 
			[4] = { 504, 504, spellLevel = 38, }, 
			[5] = { 780, 780, spellLevel = 48, }, 
			[6] = { 1044, 1044, spellLevel = 58, },
			[7] = { 1356, 1356, spellLevel = 67, }, 
			[8] = { 1440, 1440, spellLevel = 73, },
			[9] = { 1740, 1740, spellLevel = 79, }, 
		},
		[GetSpellInfo(603)] = {
			["Name"] = "Curse of Doom",
			[0] = { School = { "Shadow", "Affliction", }, sFactor = 0.5, NoSchoolTalents = true, eDot = true, eDuration = 60, },
			[1] = { 3200, 3200, spellLevel = 60, },
			[2] = { 4200, 4200, spellLevel = 70, },
			[3] = { 7300, 7300, spellLevel = 80, },
		},
		[GetSpellInfo(18220)] = {
			["Name"] = "Dark Pact",
			[0] = { School = { "Shadow", "Affliction", }, bonusFactor = 0.96, NoSchoolTalents= true, NoDPS = true, NoTooltip = true, NoAura = true, Unresistable = true, NoDownRank = true, },
			[1] = { 305, 305, spellLevel = 40, },
			[2] = { 440, 440, spellLevel = 50, },
			[3] = { 545, 545, spellLevel = 60, },
			[4] = { 700, 700, spellLevel = 70, },
			[5] = { 1200, 1200, spellLevel = 80, },
		},			
		[GetSpellInfo(6789)] = {
			["Name"] = "Death Coil",
			[0] = { School = { "Shadow", "Affliction", }, sFactor = 0.5, Cooldown = 120, Leech = true, NoDoom = true, BaseIncrease = true, LevelIncrease = 7, Downrank = 1 },
			[1] = { 244, 244, 13, 13, spellLevel = 42, },
			[2] = { 319, 319, 15, 15, spellLevel = 50, }, 
			[3] = { 400, 400, 18, 18, spellLevel = 58, },
			[4] = { 519, 519, 6, 7, spellLevel = 68, }, --guessing level increase
			[5] = { 670, 670, 1, 2, spellLevel = 73, }, --guessing level increase
			[6] = { 790, 790, 1, 2, spellLevel = 78, }, --guessing level increase
			--Note: Downrank: +1 -> exception
			--Check LevelIncrease
		},
		[GetSpellInfo(689)] = {
			["Name"] = "Drain Life",
			[0] = { School = { "Shadow", "Affliction", }, castTime=5, Leech = true, sHits = 5, sFactor = 0.5, },
			[1] = { 10, 10, spellLevel = 14, },
			[2] = { 17, 17, spellLevel = 22, },
			[3] = { 29, 29, spellLevel = 30, },
			[4] = { 41, 41, spellLevel = 38, },
			[5] = { 55, 55, spellLevel = 46, },
			[6] = { 71, 71, spellLevel = 54, },
			[7] = { 87, 87, spellLevel = 62, },
			[8] = { 108, 108, spellLevel = 69, },
			[9] = { 133, 133, spellLevel = 78, },
		},
		[GetSpellInfo(48181)] = {
			["Name"] = "Haunt",
			[0] = { School = { "Shadow", "Destruction" }, castTime = 1.5, canCrit = true, Cooldown = 8, BaseIncrease = true, },
			[1] = { 405, 473, 1, 2, spellLevel = 60, }, --guessing level increase
			[2] = { 487, 569, 1, 2, spellLevel = 70, }, --guessing level increase
			[3] = { 550, 642, 1, 2, spellLevel = 75, }, --guessing level increase
			[4] = { 645, 753, 1, 2, spellLevel = 80, }, --guessing level increase
		},
		[GetSpellInfo(1454)] = {
			["Name"] = "Life Tap",
			[0] = { School = { "Shadow", "Affliction", }, spiritFactor = 3.0, bonusFactor = 0.8, NoSchoolTalents= true, NoDPS = true, NoTooltip = true, NoAura = true, Unresistable = true, },
			[1] = { 0, 0, spellLevel = 6, spiritFactor = 1.0, },
			[2] = { 6, 6, spellLevel = 16, spiritFactor = 1.5, },
			[3] = { 24, 24, spellLevel = 26, spiritFactor = 2.0, },
			[4] = { 37, 37, spellLevel = 36, spiritFactor = 2.5, },
			[5] = { 42, 42, spellLevel = 46, },
			[6] = { 500, 500, spellLevel = 56, },
			[7] = { 710, 710, spellLevel = 68, },
			[8] = { 1490, 1490, spellLevel = 80, },
		},
		[GetSpellInfo(27243)] = { 
			["Name"] = "Seed of Corruption",
			[0] = { School = { "Shadow", "Affliction", }, canCrit = true, sFactor = 1/2, eDuration = 18, hybridDotFactor = 2 * 1.25, sTicks = 3, },
			[1] = { 1110, 1290, hybridDotDmg=1044, spellLevel = 70, },
			[2] = { 1383, 1607, hybridDotDmg=1296, spellLevel = 75, },
			[3] = { 1633, 1897, hybridDotDmg=1044, spellLevel = 80, },
		},
		[GetSpellInfo(47897)] = {
			["Name"] = "Shadowflame",
			[0] = { School = { "Shadow", "Destruction" }, canCrit = true, castTime=0, eDuration = 8, sTicks = 4, BaseIncrease = true, Cooldown = 15 },
			[1] = { 310, 338, 1, 2, spellLevel = 75,  }, --guessing level increase
			[2] = {	363, 397, 1, 2, spellLevel = 80, }, --guessing level increase
			["Secondary"] = {
				["Name"] = "Shadowflame",
				[0] = { School = { "Fire", "Destruction" }, castTime=0, eDuration = 8, sTicks = 4, Cooldown = 15 },
				[1] = { 0, 0, hybridDotDmg = 180, spellLevel = 75,  }, --guessing level increase
				[2] = {	0, 0, hybridDotDmg = 212, spellLevel = 80, }, --guessing level increase
			},
		},
		[GetSpellInfo(686)] = {
			["Name"] = "Shadow Bolt",
			[0] = { School = { "Shadow", "Destruction" }, castTime = 3, canCrit = true, CastMod = 0.1, BaseIncrease = true, LevelIncrease = 5 },
			[1] = { 12, 16, 1, 2, spellLevel = 1, castTime = 1.7, Downrank = -1, },
			[2] = { 23, 29, 3, 4, spellLevel = 6, castTime = 2.2, },
			[3] = { 48, 56, 4, 5, spellLevel = 12, castTime = 2.8, },
			[4] = { 86, 98, 6, 6, spellLevel = 20, },
			[5] = { 142, 162, 8, 8, spellLevel = 28, },
			[6] = { 204, 230, 9, 10, spellLevel = 36, },
			[7] = { 281, 315, 11, 12, spellLevel = 44, },
			[8] = { 360, 402, 13, 13, spellLevel = 52, }, 
			[9] = { 455, 507, 15, 15, spellLevel = 60, },
			[10] ={ 482, 538, 15, 16, spellLevel = 62, },
			[11] ={ 541, 603, 3, 4, spellLevel = 69, }, --guessing level increase
			[12] ={ 596, 664, 1, 2, spellLevel = 74, }, --guessing level increase
			[13] ={ 690, 770, 1, 2, spellLevel = 79, }, --guessing level increase
		},
		[GetSpellInfo(6229)] = {
			["Name"] = "Shadow Ward",
			[0] = { School = "Shadow", bonusFactor = 0.30, NoSchoolTalents= true, NoDPS = true, NoTooltip = true, NoAura = true, Unresistable = true, NoDownRank = true },
			[1] = { 290, 290, spellLevel = 32, },
			[2] = { 470, 470, spellLevel = 42, },
			[3] = { 675, 675, spellLevel = 52, },
			[4] = { 875, 875, spellLevel = 60, },
			[5] = { 2750, 2750, spellLevel = 72, },
			[6] = { 3300, 3300, spellLevel = 78, },
		},		
		[GetSpellInfo(30108)] = {
			["Name"] = "Unstable Affliction",
			[0] = { School = { "Shadow", "Affliction", }, eDot = true, eDuration = 18, sTicks = 3, canDotCrit = true, castTime = 1.5 },
			[1] = { 550, 550, spellLevel = 50, },
			[2] = { 700, 700, spellLevel = 60, },
			[3] = { 875, 875, spellLevel = 70, },
			[4] = { 985, 985, spellLevel = 75, },
			[5] = { 1150, 1150, spellLevel = 80, },
		},
		-- IMP abilities
		[GetSpellInfo(3110)] = {
			["Name"] = "Firebolt",
			[0] = { School = { "Fire", "WarlockPet", }, castTime = 2.5, canCrit = true, },
			[1] = { 6, 8, spellLevel = 1, castTime = 2, },
			[2] = { 12, 14, spellLevel = 8, castTime = 2, },
			[3] = { 22, 26, spellLevel = 18, castTime = 2, }, 
			[4] = { 33, 37, spellLevel = 28, }, 
			[5] = { 48, 54, spellLevel = 38, }, 
			[6] = { 67, 75, spellLevel = 48, },
			[7] = { 83, 93, spellLevel = 58, }, 
			[8] = { 110, 124, spellLevel = 68, },
			[9] = { 199, 223, spellLevel = 78, }, 
		},
		-- VOIDWALKER abilities (none)
		-- SUCCUBUS abilities
		[GetSpellInfo(7814)] = {
			["Name"] = "Lash of Pain",
			[0] = { School = { "Shadow", "WarlockPet", }, castTime = 0, canCrit = true, Cooldown = 12, },
			[1] = { 12, 12, spellLevel = 20, },
			[2] = { 44, 44, spellLevel = 28, },
			[3] = { 60, 60, spellLevel = 36, }, 
			[4] = { 73, 73, spellLevel = 44, }, 
			[5] = { 87, 87, spellLevel = 52, }, 
			[6] = { 99, 99, spellLevel = 60, },
			[7] = { 123, 123, spellLevel = 68, }, 
			[8] = { 193, 193, spellLevel = 74, },
			[9] = { 237, 237, spellLevel = 80, }, 
		},
		-- FELHUNTER abilities
		[GetSpellInfo(54049)] = {
			["Name"] = "Shadow Bite",
			[0] = { School = { "Shadow", "WarlockPet", }, castTime = 0, canCrit = true, Cooldown = 6, },
			[1] = { 34, 46, spellLevel = 42, },
			[2] = { 48, 66, spellLevel = 50, },
			[3] = { 73, 103, spellLevel = 58, }, 
			[4] = { 84, 118, spellLevel = 66, }, 
			[5] = { 98, 138, spellLevel = 74, }, 
		},
		-- FELGUARD abilities (pending)
		[GetSpellInfo(30213)] = {
			["Name"] = "Cleave",
			[0] = { School = { "Melee", "WarlockPet", }, castTime = 0, canCrit = true, Cooldown = 6, WeaponDamage = 1, },
			[1] = { 32, 32, spellLevel = 50, },
			[2] = { 50, 50, spellLevel = 60, },
			[3] = { 78, 78, spellLevel = 68, }, 
			[4] = { 124, 124, spellLevel = 76, }, 
		},
		-- METAMORPHOSIS abilities (pending)
		[GetSpellInfo(50589)] = {
			["Name"] = "Immolation Aura",
			[0] = { School = { "Fire", "Demonology", }, castTime = 0, eDot = true, eDuration = 15, sTicks = 15, Cooldown = 30 },
			[1] = { 6765, 6765, spellLevel = 60, },
		},
		[GetSpellInfo(50581)] = {
			["Name"] = "Shadow Cleave",
			[0] = { School = { "Shadow", "Demonology", }, castTime = 0, Cooldown = 6 }, --need to put in "on next melee" indicator
			[1] = { 110, 110, spellLevel = 60, },
		},

		-- above spells are completely updated

		[GetSpellInfo(1120)] = {
			["Name"] = "Drain Soul",
			[0] = { School = { "Shadow", "Affliction", }, castTime=15, sFactor = 0.5, sTicks = 3, NoDownRank = true, },
			[1] = { 55, 55, spellLevel = 10, },
			[2] = { 155, 155, spellLevel = 24, },
			[3] = { 295, 295, spellLevel = 38, },
			[4] = { 455, 455, spellLevel = 52, },
			[5] = { 620, 620, spellLevel = 67, }, 
		},
		[GetSpellInfo(18265)] = {
			["Name"] = "Siphon Life",
			[0] = { School = { "Shadow", "Affliction", }, sHits = 10, Leech = true, eDot = true, eDuration = 30, sFactor = 0.5, Downrank = 5 },
			[1] = { 15, 15, spellLevel = 30, Downrank = 3 },
			[2] = { 22, 22, spellLevel = 38, },
			[3] = { 33, 33, spellLevel = 48, },
			[4] = { 45, 45, spellLevel = 58, },
			[5] = { 52, 52, spellLevel = 63, },
			[6] = { 63, 63, spellLevel = 70, },
			--Note: Downranking +3/5, why?
		},
		[GetSpellInfo(5676)] = {
			["Name"] = "Searing Pain",
			[0] = { School = { "Fire", "Destruction" }, canCrit = true, BaseIncrease = true, LevelIncrease = 7, Downrank = 1 },
			[1] = { 34, 42, 4, 5, spellLevel = 18, },
			[2] = { 59, 71, 6, 6, spellLevel = 26, },
			[3] = { 86, 104, 7, 8, spellLevel = 34, },
			[4] = { 122, 146, 9, 9, spellLevel = 42, },
			[5] = { 158, 188, 10, 10, spellLevel = 50, },
			[6] = { 204, 240, 12, 12, spellLevel = 58, },
			[7] = { 243, 287, 9, 10, spellLevel = 65, },
			[8] = { 270, 320, 0, 0, spellLevel = 70, },
			--Note: Downranking +1 -> exception
			--Check LevelIncrease
		},
		[GetSpellInfo(6353)] = {
			["Name"] = "Soul Fire",
			[0] = { School = { "Fire", "Destruction" }, bonusFactor = 1.15, castTime=6, canCrit = true, CastMod = 0.4, BaseIncrease = true, Cooldown = 60,  LevelIncrease = 7 },
			[1] = { 623, 783, 17, 18, spellLevel = 48, },
			[2] = { 703, 881, 18, 19, spellLevel = 56, },
			[3] = { 839, 1051, 14, 14, spellLevel = 64, },
			[4] = { 1003, 1257, 0, 0, spellLevel = 70, },
			--Check LevelIncrease
		},
		[GetSpellInfo(17877)] = {
			["Name"] = "Shadowburn",
			[0] = { School = { "Shadow", "Destruction" }, Cooldown = 15, canCrit = true, BaseIncrease = true,  LevelIncrease = 7 },
			[1] = { 87, 99, 4, 6, spellLevel = 20, },
			[2] = { 115, 131, 8, 9, spellLevel = 24, },
			[3] = { 186, 210, 10, 11, spellLevel = 32, },
			[4] = { 261, 293, 13, 14, spellLevel = 40, },
			[5] = { 350, 392, 15, 16, spellLevel = 48, },
			[6] = { 450, 502, 17, 19, spellLevel = 56, },
			[7] = { 518, 578, 20, 21, spellLevel = 63, },
			[8] = { 597, 665, 0, 0, spellLevel = 70, },
			--Check LevelIncrease
		},
		[GetSpellInfo(348)] = {
			["Name"] = "Immolate",
			[0] = { School = { "Fire", "Destruction" }, canCrit = true, castTime=2, CastMod = 0.1, hybridFactor = 0.349, eDuration = 15, sTicks = 3, BaseIncrease = true,  LevelIncrease = 5 },
			[1] = { 8, 8, 2, 2, hybridDotDmg = 20, spellLevel = 1, Downrank = -1 },
			[2] = {	19, 19, 4, 4, hybridDotDmg = 40, spellLevel = 10, }, 	 
			[3] = { 45, 45, 7, 7, hybridDotDmg = 90, spellLevel = 20, },	 
			[4] = { 90, 90, 10, 10, hybridDotDmg = 165, spellLevel = 30, }, 
			[5] = { 134, 134, 13, 13, hybridDotDmg = 255, spellLevel = 40, },
			[6] = { 192, 192, 16, 16, hybridDotDmg = 365, spellLevel = 50, }, 
			[7] = { 258, 258, 19, 19, hybridDotDmg = 485, spellLevel = 60, },
			[8] = { 279, 279, 19, 20, hybridDotDmg = 510, spellLevel = 60, },
			[9] = { 327, 327, 4, 5, hybridDotDmg = 615, spellLevel = 69, },
		},
		[GetSpellInfo(1949)] = {
			["Name"] = "Hellfire",
			[0] = { School = "Fire", castTime=15, sHits = 15, sFactor = 1/2, BaseIncrease = true, LevelIncrease = 11, Downrank = 5, },
			[1] = { 83, 83, 4, 4, spellLevel = 30, },
			[2] = { 139, 139, 5, 5, spellLevel = 42, },
			[3] = { 208, 208, 7, 7, spellLevel = 54, },
			[4] = { 306, 306, 1, 2, spellLevel = 68, },
			--Note: Downrank +5 -> exception
		},
		[GetSpellInfo(5740)] = {
			["Name"] = "Rain of Fire",
			[0] = { School = "Fire", castTime=8, bonusFactor = 1.1458, sTicks = 2, BaseIncrease = true, LevelIncrease = 11, },
			[1] = { 168, 168, 4, 8, spellLevel = 20, },
			[2] = { 384, 384, 8, 8, spellLevel = 34, },
			[3] = { 620, 620, 8, 12, spellLevel = 46, },
			[4] = { 904, 904, 12, 12, spellLevel = 58, },
			[5] = { 1212, 1212, 0, 4, spellLevel = 69, },
			--Check LevelIncrease
		},
		[GetSpellInfo(17962)] = { 
			["Name"] = "Conflagrate",
			[0] = { School = { "Fire", "Destruction" }, canCrit = true, BaseIncrease = true, LevelIncrease = 5, },
			[1] = { 240, 306, 9, 10, spellLevel = 40, },
			[2] = { 316, 396, 10, 11, spellLevel = 48, },
			[3] = { 383, 479, 12, 12, spellLevel = 54, },
			[4] = { 447, 557, 8, 9, spellLevel = 60, },
			[5] = { 512, 638, 9, 10, spellLevel = 65, },
			[6] = { 579, 721, 0, 0, spellLevel = 70, },
		},
		[GetSpellInfo(30283)] = { 
			["Name"] = "Shadowfury",
			[0] = { School = { "Shadow", "Destruction" }, canCrit = true, sFactor = 1/2 * 0.90, Cooldown = 20, BaseIncrease = true,  LevelIncrease = 9, },
			[1] = { 343, 407, 14, 15, spellLevel = 50, },
			[2] = { 459, 547, 17, 18, spellLevel = 60, },
			[3] = { 612, 728, 0, 0, spellLevel = 70, },
		},
		[GetSpellInfo(32231)] = { 
			["Name"] = "Incinerate",
			[0] = { School = { "Fire", "Destruction" }, canCrit = true, castTime = 2.5, BaseIncrease = true, LevelIncrease = 5, },
			[1] = { 403, 467, 13, 13, incinerateMin = 100, incinerateMax = 116, spellLevel = 64, },
			[2] = { 444, 514, 0, 0, incinerateMin = 111, incinerateMax = 128, spellLevel = 70, },
		},			
	}
	self.talentInfo = {
		--AFFLICTION:
		--Improved Curse of Agony
		[GetSpellInfo(18827)] = { 	[1] = { Effect = 0.05, Spells = "Curse of Agony", Add = true, }, },
		--Suppression
		[GetSpellInfo(18174)] = {
			[1] = { Effect = 1, Spells = "Affliction", ModType = "hitPerc", },
			[2] = { Effect = -0.02, Spells = "Affliction", ModType = "manaCostM", },
		},
		--Improved Corruption
		[GetSpellInfo(17810)] = {
			[1] = { Effect = 0.02, Spells = "Corruption", Add = true, },
			[2] = { Effect = 1, Spells = "Seed of Corruption", ModType = "critPerc", },
		},
		--Improved Life Tap
		[GetSpellInfo(18182)] = { [1] = { Effect = 0.10, Spells = "Life Tap", }, },		
		--Empowered Corruption
		[GetSpellInfo(32381)] = { [1] = { Effect = 0.12, Spells = "Corruption", ModType = "SpellDamage", }, },
		--Shadow Mastery
		[GetSpellInfo(18271)] = { [1] = { Effect = 1, Spells = { "Corruption", "Curse of Agony", "Curse of Doom", "Drain Life", "Drain Soul", "Seed of Corruption", "Siphon Life", "Unstable Affliction" }, ModType = "Amount", Value = "Shadow Embrace" }, },			
		--Soul Siphon
		[GetSpellInfo(17804)] = { [1] = { Effect = 1, Spells = { "Drain Life", "Drain Soul" }, ModType = "Amount", Value = "Soul Siphon" }, },
--		--Improved Felhunter --only needed when we are factoring in mana usage
--		[GetSpellInfo(54037)] = { [1] = { Effect = 1, Spells = "All", ModType = "Amount", Value = "Improved Felhunter" }, },
--		--Eradication --hard to model this effect on spells currently
--		[GetSpellInfo(47195)] = { [1] = { Effect = 1, Spells = "All", ModType = "Amount", Value = "Eradication" }, },			
		--Contagion
		[GetSpellInfo(30060)] = { [1] = { Effect = 0.01, Spells = { "Corruption", "Curse of Agony", "Seed of Corruption" }, Add = true, }, },
		--Malediction
		[GetSpellInfo(32477)] = { [1] = { Effect = 0.01, Spells = "All", Add = true, }, },
		--Death's Embrace
		[GetSpellInfo(47198)] = { [1] = { Effect = 1, Spells = "All", ModType = "Amount", Value = "Death's Embrace" }, },			
		--Pandemic
		[GetSpellInfo(58435)] = { [1] = { Effect = 1, Spells = { "Corruption", "Unstable Affliction" }, ModType = "Amount", Value = "Pandemic" }, },
		--Everlasting Affliction
		[GetSpellInfo(47201)] = { [1] = { Effect = 0.01, Spells = { "Corruption", "Siphon Life", "Unstable Affliction" }, ModType = "SpellDamage", }, },			

		--DEMONOLOGY:
		--Improved Imp
		[GetSpellInfo(18694)] = { [1] = { Effect = 0.1, Spells = "Firebolt", Add = true, }, },
		--Demonic Brutality
		[GetSpellInfo(18705)] = { [1] = { Effect = 1, Spells = "Cleave", ModType = "Amount", Value = "Demonic Brutality" }, },
		--Demonic Aegis
		[GetSpellInfo(30143)] = { [1] = { Effect = 1, Spells = "All", ModType = "Amount", Value = "Demonic Aegis" }, },
		--Unholy Power
		[GetSpellInfo(18769)] = { [1] = { Effect = 0.04, Spells = { "Cleave", "Firebolt", "Lash of Pain", "Shadow Bite" }, }, },
		--Master Conjuror
		[GetSpellInfo(18767)] = { [1] = { Effect = 1, Spells = "All", ModType = "Amount", Value = "Master Conjuror" }, },
		--Master Demonologist
		[GetSpellInfo(23785)] = { [1] = { Effect = 1, Spells = "All", ModType = "Amount", Value = "Master Demonologist" }, },
		--Demonic Tactics --don't forget to make this apply to pet abilities later
		[GetSpellInfo(30242)] = { [1] = { Effect = 1, Spells = "All", ModType = "critPerc" }, },
--		--Fel Synergy --extra pet intellect means extra pet critical strike chance
--		[GetSpellInfo(47230)] = { [1] = { Effect = 1, Spells = "All", ModType = "Amount", Value = "Fel Synergy" }, },
--		--Improved Demonic Tactics --extra pet critical strike chance has consequences for Demonic Empathy and Demonic Pact uptime
--		[GetSpellInfo(48079)] = { [1] = { Effect = 1, Spells = "All", ModType = "Amount", Value = "Demonic Empathy" }, },
--		--Demonic Pact --needs pet +crit detection in order to be remotely accurate
--		[GetSpellInfo(47236)] = { [1] = { Effect = 1, Spells = "All", ModType = "Amount", Value = "Demonic Pact" }, },
		
		--DESTRUCTION:
		--Improved Shadow Bolt
		[GetSpellInfo(17793)] = { [1] = { Effect = 1, Spells = "Shadow Bolt", ModType = "Amount", Value = "Improved Shadow Bolt" }, },
--		--Bane
		[GetSpellInfo(17788)] = { 
			[1] = { Effect = 1, Mod = { "Chaos Bolt", 3, 2, "castTime", function(v,c,t) return (v-c*0.1) end }, },
			[2] = { Effect = 1, Mod = { "Immolate", 3, 2, "castTime", function(v,c,t) return (v-c*0.1) end }, },
			[3] = { Effect = 1, Mod = { "Shadow Bolt", 3, 2, "castTime", function(v,c,t) return (v-c*0.1) end }, },
			[4] = { Effect = 1, Mod = { "Soul Fire", 3, 2, "castTime", function(v,c,t) return (v-c*0.4) end }, },
		},
		--Cataclysm
		[GetSpellInfo(17778)] = {
			[1] = { Effect = 1, Spells = "Destruction", ModType = "hitPerc", },
			[2] = { Effect = -0.01, Spells = "Destruction", ModType = "manaCostM", },
		},
		--Demonic Power
		[GetSpellInfo(18126)] = {
			[1] = { Effect = 1, Spells = "Lash of Pain", Mod = { "Firebolt", 3, 6, "Cooldown", function(v,c,t) return (v-c*3) end },},
			[2] = { Effect = 1, Spells = "Firebolt", Mod = { "Firebolt", 3, 6, "castTime", function(v,c,t) return (v-c*0.25) end },},
		},
		--Ruin
		[GetSpellInfo(17959)] = { [1] = { Effect = 0.1, Spells = "Destruction", ModType = "critM", }, },
		--Improved Searing Pain
		[GetSpellInfo(17927)] = { [1] = { Effect = { 4, 7, 10, }, Spells = "Searing Pain", ModType = "critPerc", }, },
		--Improved Immolate
		[GetSpellInfo(17815)] = { [1] = { Effect = 0.1, Spells = "Immolate", ModType = "bNukeDmg" }, },
		--Devastation
		[GetSpellInfo(18130)] = { [1] = { Effect = 5, Spells = "Destruction", ModType = "critPerc", }, },
		--Shadow and Flame
		[GetSpellInfo(30288)] = { [1] = { Effect = 0.04, Spells = { "Shadow Bolt", "Incinerate" }, ModType = "SpellDamage", }, },
		--Emberstorm
		[GetSpellInfo(17954)] = { 
			[1] = { Effect = 1, Spells = "Incinerate", Mod = { "Incinerate", 3, 16, "castTime", function(v,c,t) return (v-c*0.05) end }, },
			[2] = { Effect = 0.02, Spells = "Fire", Add = true, }, 
		},
		--Backlash
		[GetSpellInfo(34935)] = { [1] = { Effect = 1, Spells = "All", ModType = "critPerc", }, },
--		--Improved Soul Leech --could be useful for mana regain calculation
--		[GetSpellInfo(54117)] = { [1] = { Effect = 1, Spells = "All", ModType = "Amount", Value = "Improved Soul Leech" }, },
--		--Backdraft --could be useful for haste calculations assuming a rotation
--		[GetSpellInfo(47258)] = { [1] = { Effect = 1, Spells = "All", ModType = "Amount", Value = "Backdraft" }, },
		--Fire and Brimstone
		[GetSpellInfo(47266)] = { 
			[1] = { Effect = 1, Spells = "Conflagrate", ModType = "Amount", Value = "Fire and Brimstone", }, --not yet being calculated
			[2] = { Effect = 0.03, Spells = "Immolate", ModType = "SpellDamage", }, 
		},
	}
end